@@ -27,7 +27,7 @@ print('\n\n-------\nSTART:\n-------\n\n')
 
 
 def generate_list_of_pomidor_files(tomato_directory):
-    """Go through a gived directory and create a list of filenames with
+    """Goes through a gived directory and creates a list of filenames with
     .pomidor extension"""
 
     tomato_files_list = []
@@ -50,14 +50,14 @@ def generate_list_of_pomidor_files(tomato_directory):
 def define_test_paragraphs(scenarioSteps, filepath, frst_prgrph_line,
                            scenario_title_line_num, line_num,
                            obj_dict, driver, url, wait):
-    """Go over a particular test case paragraph and execute all actions"""
+    """Goes over a particular test case paragraph and executes all actions"""
 
     latest_index = 0
     action_counter = 0
     obj_counter = 0
     tied_obj = False
     scenario_with_action = False
-    str_in_quotes = re.findall(r" \'(.+?)\'", scenarioSteps)
+    str_in_quotes = re.findall(r"\'(.+?)\'", scenarioSteps)
     str_in_brackets = re.findall(r" \[(.+?)\]", scenarioSteps)
     print(f'str_in_brackets --> {str_in_brackets}')
     print(f"str_in_quotes --> {str_in_quotes} ")
@@ -79,9 +79,10 @@ def define_test_paragraphs(scenarioSteps, filepath, frst_prgrph_line,
                 backward_action_dict = bact.backward_actions_dictionary
                 forward_action_dict = act.forward_action_dictionary
                 print(f'FOWRD dICT --> {forward_action_dict}')
-
                 # Iterate over forward actions
                 for action_in_bckwrd_list in backward_action_dict.keys():
+                    if action_in_bckwrd_list == '*page_title':
+                        pass
                     # look for backward action first
                     print(
                         f'SUCCESS -> for action_in_bckwrd_list in '
@@ -221,12 +222,20 @@ def define_test_paragraphs(scenarioSteps, filepath, frst_prgrph_line,
                     # if page_object_src not in
                     print(f'Latest index --> {latest_index}')
                     print(f"\nActions and Assertions performed:")
-                    if not browser_initialized and \
-                            Pomidor.before_tests_launch_url.has_been_called:
+                    if not browser_initialized:
+                        # and Pomidor.before_tests_launch_url.has_been_called:
                         pomidor = Pomidor(driver, obj_dict, url)
-                        pom_driver = pomidor.define_browser()
+                        driver = pomidor.define_browser()
                         browser_initialized = True
-                        pom_driver.get(url)
+                        driver.get(url)
+                        driver.title
+                        # driver.
+                        if Pomidor.max_window.has_been_called:
+                            driver.maximize_window()
+                        if Pomidor.fullscreen.has_been_called:
+                            driver.fullscreen_window()
+                        if Pomidor.delete_all_cookies.has_been_called:
+                            driver.delete_all_cookies()
 
                     # if forward or backward action
                     if backward_action:
@@ -240,7 +249,18 @@ def define_test_paragraphs(scenarioSteps, filepath, frst_prgrph_line,
                                                            wait)
                     print(f'ZZZZ! -> act_func : {act_func}')
                     print(f'str_in quotes -> {str_in_quotes}')
-                    print(pom_driver)
+                    print(driver)
+                    if act == 'send_keys()':
+                        exec(f'WebDriverWait(driver, '
+                             f'{wait}).until(ec.element_to_be_clickable('
+                             f'(By.{page_obj_locator}, '
+                             f'\"{page_object_src}\"))).clear()')
+                        # TODO add test verifying the content of send_keys()
+                        # TODO add *text_value_is [of #page_obj] or #page_obj has *text_value of 'Blah'
+                        # TODO add *page_title_is 'Practice - Pomidor Auto...'
+                        # TODO create test paragraph skeleton - all actions are enumerated and
+                        #  appropriate StrList index is used
+                        time.sleep(1)
                     exec(act_func)
             elif strList_item.startswith('#') and not tied_obj:
                 obj_counter += 1
@@ -268,16 +288,43 @@ def define_test_paragraphs(scenarioSteps, filepath, frst_prgrph_line,
                     f'{scenario_title_line_num}')
         print('SCENARIO COMPLETED!!!!')
     finally:
-        if Pomidor.quit.has_been_called and browser_initialized:
-            time.sleep(1)
-            pom_driver.quit()
+        if browser_initialized:
+            # Pomidor.quit.has_been_called and :
+            # time.sleep(1)
+            driver.quit()
             print('\nDriver QUIT!!\n')
         pass
     return scenario_with_action
 
 
+def pomidor_pro_func(code_string, driver, obj_dict, url):
+    """Not used..."""
+
+    try:
+        # and Pomidor.before_tests_launch_url.has_been_called:
+        pomidor = Pomidor(driver, obj_dict, url)
+        driver = pomidor.define_browser()
+        browser_initialized = True
+        driver.get(url)
+        driver.title
+        # driver.
+        if Pomidor.max_window.has_been_called:
+            driver.maximize_window()
+        if Pomidor.fullscreen.has_been_called:
+            driver.fullscreen_window()
+        if Pomidor.delete_all_cookies.has_been_called:
+            driver.delete_all_cookies()
+    finally:
+        if browser_initialized:
+            # Pomidor.quit.has_been_called and :
+            # time.sleep(1)
+            driver.quit()
+            print('\nDriver QUIT!!\n')
+        pass
+
+
 def go_thru_pomidor_file(func, obj_dict, driver, base_url, urls, wait):
-    """Open one .pomidor file at a time and pick test case paragraphs,  one by
+    """Opens one .pomidor file at a time and picks test case paragraphs, one by
     one, top to bottom"""
 
     scenario_number = 0
@@ -330,9 +377,9 @@ def go_thru_pomidor_file(func, obj_dict, driver, base_url, urls, wait):
 def go_thru_pomidor_file_with_story(func, feature_type, story, obj_dict,
                                     driver, base_url, urls,
                                     exact_story_name, wait):
-    """Open one .pomidor file at a time and pick test case paragraphs marked
-    with a passed @marker value (Ex."@story", "@feature" or your own custom
-    marker, one by one, top to bottom"""
+    """Opens a .pomidor file, one at a time, and picks test case paragraphs
+    marked with a passed @marker value (Ex."@story", "@feature" or your own
+    custom marker, one by one, top to bottom"""
 
     scenario_number = 0
     for file_number, filepath in enumerate(func):
@@ -346,12 +393,16 @@ def go_thru_pomidor_file_with_story(func, feature_type, story, obj_dict,
             feature_instances = 0
             line_counter = 0
             url = base_url
+            pro = False
+            first_pro_line = False
+            browser_initialized = False
             for line_num, line in enumerate(tomato_file):
                 if line == 0:
                     continue
                 else:
                     line_counter += 1
                 print(f'======= ===== General Line #{line_num}====== ======')
+                # choose type of marker to run
                 if '@base_url' in line:
                     line_list = re.split(r'[;,.!?\s]', line)
                     ad_hoc_url = line_list[1]
@@ -372,49 +423,99 @@ def go_thru_pomidor_file_with_story(func, feature_type, story, obj_dict,
                                 print(f'$@$$ FOUND {feature_type}! -> {f}\n')
                                 print(f'Line before inner loop--> {line}')
                                 feature_instances += 1
-                    if feature_instances > 0:
-                        for line_num_in, line in enumerate(tomato_file,
-                                                           line_counter):
-                            line_counter += 1
-                            print(f'Text "{line}" is on line num --> '
-                                  f'{line_num_in + 1}')
+                if '@data' in line:
+                    line_list = re.split(r'[;,.!?\s]', line)
+                    ad_hoc_url = line_list[1]
+                    url = urls.get(ad_hoc_url)
+                    print(f'@data is caught -> ! {url}')
+                if feature_instances > 0:
+                    for line_num_in, line in enumerate(tomato_file,
+                                                       line_counter):
+                        line_counter += 1
+                        print(f'Text "{line}" is on line num --> '
+                              f'{line_num_in + 1}')
 
-                            # if feature_bool:
-                            if scenarioSteps == '' and (line in ['\n', '\r\n']
-                                                        or line.startswith(
-                                        '--')):  # .pomidor comment
+                        # if feature_bool:
+                        if scenarioSteps == '' and (line in ['\n', '\r\n']
+                                                    or line.startswith(
+                                    '--')):  # .pomidor comment
+                            continue
+                        else:
+                            if not pro and line.startswith('"""'):
+                                print("@pro is found!")
+                                pro = True
+                                first_pro_line = True
+                            if scenarioSteps == '':
+                                print("after @pro is found!")
+                                first_paragraph_line = line
+                                scenario_title_line_num = line_num
+                            if scenarioSteps != '' and line.startswith(
+                                    '--'):
                                 continue
+                            if pro:
+                                if first_pro_line:
+                                    first_pro_line = False
+                                else:
+                                    print("@not""")
+                                    scenarioSteps = scenarioSteps + line + '\n'
+                                    print(f'!!scenarioSteps-> {scenarioSteps}')
                             else:
-                                if scenarioSteps == '':
-                                    first_paragraph_line = line
-                                    scenario_title_line_num = line_num
-                                if scenarioSteps != '' and line.startswith(
-                                        '--'):
-                                    continue
                                 scenarioSteps += line
-                            if (scenarioSteps != '' and line in ['\n', '\r\n']) \
-                                    or (
-                                    scenarioSteps != '' and line_counter - 1 ==
-                                    total_lines_count + 1):
-                                print(f'total_lines_count:{total_lines_count}')
-                                print(f'LINE IS -----> {line} at '
-                                      f'{line_num_in}')
-                                print(
-                                    f"\nBegin test:\n ----"
-                                    f"{first_paragraph_line.strip()}----\n"
-                                    f"\nActions and Assertions performed:")
-                                latest_index = 0
-                                action_counter = 0
-                                test_paragraph = define_test_paragraphs(
-                                    scenarioSteps, filepath,
-                                    first_paragraph_line,
-                                    scenario_title_line_num,
-                                    line_num, obj_dict, driver, url, wait)
-                                if test_paragraph:
-                                    scenario_number += 1
-                                scenarioSteps = ''
-                                feature_instances = 0
-                                break
+                                print("Captured wrong scenarioSteps!!")
+                        if pro and (scenarioSteps != ''
+                                     and line.startswith('"""')):
+                            scenarioSteps = scenarioSteps.strip('\n"""')
+                            print(f'PROscenarioSteps --> {scenarioSteps}')
+                            try:
+                                if not browser_initialized:
+                                    # and Pomidor.before_tests_launch_url.has_been_called:
+                                    pomidor = Pomidor(driver, obj_dict, url)
+                                    driver = pomidor.define_browser()
+                                    browser_initialized = True
+                                    if Pomidor.max_window.has_been_called:
+                                        driver.maximize_window()
+                                    if Pomidor.fullscreen.has_been_called:
+                                        driver.fullscreen_window()
+                                    if Pomidor.delete_all_cookies.has_been_called:
+                                        driver.delete_all_cookies()
+                                    exec(scenarioSteps)
+                                    time.sleep(1)
+                            finally:
+                                if browser_initialized:
+                                    # browser_initialized = False
+                                    # Pomidor.quit.has_been_called and :
+                                    # time.sleep(1)
+                                    driver.quit()
+                                    print('\nDriver QUIT!!\n')
+                                pass
+                            pro = False
+                            scenario_number += 1
+                            scenarioSteps = ''
+                            feature_instances = 0
+                            break
+                        if not pro and (scenarioSteps != ''
+                                        and line in ['\n', '\r\n']) or (
+                                scenarioSteps != '' and line_counter - 1 ==
+                                total_lines_count + 1):
+                            print(f'total_lines_count:{total_lines_count}')
+                            print(f'LINE IS -----> {line} at '
+                                  f'{line_num_in}')
+                            print(
+                                f"\nBegin test:\n ----"
+                                f"{first_paragraph_line.strip()}----\n"
+                                f"\nActions and Assertions performed:")
+                            latest_index = 0
+                            action_counter = 0
+                            test_paragraph = define_test_paragraphs(
+                                scenarioSteps, filepath,
+                                first_paragraph_line,
+                                scenario_title_line_num,
+                                line_num, obj_dict, driver, url, wait)
+                            if test_paragraph:
+                                scenario_number += 1
+                            scenarioSteps = ''
+                            feature_instances = 0
+                            break
                     else:
                         line_counter += 1
                         url = base_url
@@ -459,14 +560,13 @@ def list_all_mark_values(func, feature_type):
                           f'{marker_values}++++++++++\n')
     return marker_values, len(marker_values)
 
-
 # Selenium action functions:
 
 def action_func_visible(driver, act, obj_source, locator, wait):
     """Function to construct a string with expected condition:
     visibility_of_element_located"""
 
-    return f'WebDriverWait(pom_driver, {wait}).until(ec.visibility_of_' \
+    return f'WebDriverWait(driver, {wait}).until(ec.visibility_of_' \
            f'element_located((By.{locator},\"{obj_source}\"))).{act}'
 
 
@@ -489,7 +589,8 @@ def action_func_clickable(act, obj_source, locator, str_list, wait):
     if act == "click()" or act == "send_keys()":
         if act == "send_keys()":
             act, str_list = send_keys_func(str_list)
-    return f'WebDriverWait(pom_driver, {wait}).until(ec.element_to_be_' \
+
+    return f'WebDriverWait(driver, {wait}).until(ec.element_to_be_' \
            f'clickable((By.{locator}, \"{obj_source}\"))).{act}', str_list
 
 
@@ -559,6 +660,18 @@ class Pomidor:
     def quit(self):
         pass
 
+    @trackcalls
+    def max_window(self):
+        pass
+
+    @trackcalls
+    def fullscreen(self):
+        pass
+
+    @trackcalls
+    def delete_all_cookies(self):
+        pass
+
     def run_scripts(self, dir_path, verbose=True, wait=10):
         file_num, scenario_number = go_thru_pomidor_file(
             generate_list_of_pomidor_files(dir_path), self.obj_dict,
@@ -635,4 +748,4 @@ class Pomidor:
 
     @staticmethod
     def get_dict_obj(obj_key):  # Needed to print only
-        pass
+        pass
\ No newline at end of file
